; i8748 (1816VE48) Firmware for ES1840.C000 - Keyboard for ES1840 and ES1841
;
; i8748 Pins 12-19        = Port P0 Bits0-3 = Kbd Column Address, Bits 4-6 = Kbd Row Select
; i8748 Pins 27-34        = Port P1 Bit0 = Sync IN, Bit1 = Data IN, Bit5 = Rus LED, Bit6 = Lat LED, Bit7 = Cif LED
; i8748 Pins 21-24, 35-38 = Port P2 Bit0 = ~Kbd Matrix Scan Enable, Bit1 = ~Sync OUT, Bit2 = ~Data OUT
; i8748 Pin  39           = CD2 (T1) = ~Selected Key Input

; r0* = RAM Offset 24d = 18h	; r4* = RAM Offset 28d = 1Ch
; r1* = RAM Offset 25d = 19h    ; r5* = RAM Offset 29d = 1Dh
; r2* = RAM Offset 26d = 1Ah    ; r6* = RAM Offset 30d = 1Eh
; r3* = RAM Offset 27d = 1Bh    ; r7* = RAM Offset 31d = 1Fh

.equ	Reg_r3a		$1B	; Register r3* - Offset in RAM				; Same offset as register r3* - do not use r3* - r7*
.equ	PresRelMatr	$1C 	; [12 Bytes] PressRelease BitMap Offset in RAM (1-Pres,0-Rels), 93 Keys used 93/8+1 = 12 Bytes, allign to 16
.equ	StickKeysArr	$2C 	; [19 Bytes] Sticked Keys Scan Codes Array [2C-3F]	; MAYBE THIS IS BUFFER FOR NORMAL KEY, NEED CHECK !!!
;
; === Page0ROM (256 Bytes) ===	; === Page0ROM (256 Bytes) ===
;
	.org	0		; org 0 (Start on reset)
Start:
	clr	f0		; Clear f0 (flag0)
	stop	tcnt		; Stop Timer / Counter
	jtf	Begin	;.org 3	; org 03 (Interrupt) -> inside jtf instruction, because Interrupt not used here
	jmp	Begin		; Go to real Begin
;
	nop			; nop
;
	.org	7		; org 7 (Timer/Counter)
;
TimerStart:			; Timer routine start
	sel	rb1		; Select RAM Bank0 (registers with * and classic RAM)
	jmp	TimerBegin	; Go to real timer service routine Begin
;
Begin:				; Real Begin
	clr	f1		; Clear f1 (flag1)
	clr	a		; a = 0
	mov	r6,a		; r6 = 0
	mov	r7,a		; r7 = 0
CalcCRC1:
	movp	a,@a		; a = CurPageROM[a]
	add	a,r6		; a = a + r6
	mov	r6,a		; r6 = a         <= r6 = CRC
	inc	r7		; r7 = r7 + 1
	mov	a,r7		; a = r7
	jnz	CalcCRC1	; Loop
	jmp	Phase2CRC	; Go to next CRC calculation phase in Page1
;
; r1 = r6 = r7 = f0 = 0 here	; r1 = r6 = r7 = f0 = 0 here
ReadHostPrepare:
	anl	p1,#$3F		; Sync IN = X, Data IN = X | LED : RX LX CX
	jmp	ReadHostLoop	; Go to ReadHostLoop (General loop)
;
	nop			; Allignment
	nop			; by
	nop			; nops
;
ErrSyncHi2:			; ErrSyncHi
	jmp	ErrSyncHi	; Jump helper
;
ReadHostLoop:			; General Loop
	orl	p2,#$03		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = X | ========> K1 S1 DX
	in	a,p1		; a = Port P1 Bit0 = Sync IN, Bit1 = Data IN, Bit5 = Rus LED, Bit6 = Lat LED, Bit7 = Cif LED
	cpl	a		; a = NOT a
	jb0	ErrSyncHi2	; Bit0 (Sync IN) = NOT 1 ? ErrSyncHi2
	jf1	DoSndStickKeys	; Send Stickey Keys to Host
ChkTmrOverflow:
	jtf	TimerStart	; Timer overflow ? => TimerStart Again
KbdScanCycleT:
	inc	r7		; Next scan cycle from timer
	mov	a,r7		; Max scan cycle from timer ?
	jb7	DoNewScanCycleT	; Reset scan cycle from timer  = new scan cycle from timer
	jmp	ContinScanCyclT	; Continue scan cycle from timer
;
DoNewScanCycleT:		; Next Scan Cycle by Timer
	mov	r7,#$01		; Keyboard scan cycle from timer reset to 1
ContinScanCyclT:
	mov	a,r7		; Get scan cycle from timer
	outl	bus,a		; Out it to bus
	;
	; Enable keyboard scan	; Enable keyboard scan
	;
	anl	p2,#$42		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 0, Bit1 = ~Sync OUT = X, Bit2 = ~Data OUT = 0 | ========> K0 SX D0
	mov	a,r7		; Get scan cycle from timer
	mov	r1,a		; r1 = scan cycle
	;
	; Key press detecting	; Key press detecting
	;
	jnt1	KeyPressed	; If T1 is zero - Key pressed detected
	;
	; No key pressed	; No key pressed
	;
	call	GetBitByteIndex	; Get Byte index (offset) in PressRelease Bit Matrix in RAM for scan cycle depended key(r1) + bit number(r5)
	mov	a,@r1		; a = Byte index (offset) in PressRelease Bit Matrix in RAM	; a = RAM[r1]
	anl	a,r5		; Select according key in PressRelease Bit Matrix in RAM by masking all bits except our (r5)
	jnz	KeyFormerPresd	; If not zero - Key mark as pressed in PressRelease Bit Matrix in RAM
	;
	; Key no pressed and in PressRelease Bit Matrix in RAM marked as Released
	;
	mov	a,r7		; Get scan cycle from timer
	xrl	a,r6		; xor it with scan cycle from main
	jnz	ReadHostLoop	; Continue general loop
	stop	tcnt		; Stop Timer/Counter
	clr	f0		; f0 = 0 (Repeat key send no needed, key was released)
	mov	r0,#Reg_r3a	; Get r3* address (Used in Timer)
	clr	a		; a = 0
	mov	@r0,a		; Clear r3* (Used in Timer)	; r3* = a	; RAM[r0] = a
ClearR6AndBack:
	mov	r6,#$00		; Reset scan cycle from main
	jmp	SendReleasedKey	; Send released key Release Code to Host
;
	nop			; Allignment
	nop			; by nops
;
KeyFormerPresd:			; Key now released and marked in PressRelease Bit Matrix in RAM as pressed
	;
	; Primitive Release Debouncing as Delay 3xFF
	;
	mov	r4,#$FF		; Max value
LoopFF_01:
	djnz	r4,LoopFF_01	; r4--, loop while r4 != 0
	mov	r4,#$FF
LoopFF_02:
	djnz	r4,LoopFF_02	; r4--, loop while r4 != 0
	mov	r4,#$FF
LoopFF_03:
	djnz	r4,LoopFF_03	; r4--, loop while r4 != 0
;
	jnt1	ReadHostLoop	; If T1 is zero after delay - key was bounced, false release detected, continue as not released
	;
	; Key is really released, debounced procedure confirm this. Do mark it as released.
	;
	mov	a,@r1		; a = Byte index (offset) in PressRelease Bit Matrix in RAM	; a = RAM[r1]
	xrl	a,r5		; Mark according key into PressRelease Bit Matrix in RAM as Released by inverting (xor) our bit (r5)
	mov	@r1,a		; Store it back into PressRelease Bit Matrix in RAM		; RAM[r1] = a
	mov	a,r7		; Get scan cycle from timer
	xrl	a,r6		; xor it with scan cycle from main
	jz	ClearR6AndBack	; Do clear r6 and return back
SendReleasedKey:
	mov	a,r7		; Get scan cycle from timer
	orl	a,#$80		; Set Bit7 => This is Release Code to Host
	mov	r3,a		; Move it to r3
	jmp	TranslateToHost	; Translate Scan cycle in r3 register into Host Code. Bit7 is Lo for Press Code and Hi to Release Code.
;
KeyPressed:			; Key pressed
	;
	; Primitive Press Debouncing and also Repeat Test as Delay 3xFF
	;
	mov	r4,#$FF		; Max value
LoopFF_04:
	djnz	r4,LoopFF_04	; r4--, loop while r4 != 0
	mov	r4,#$FF
LoopFF_05:
	djnz	r4,LoopFF_05	; r4--, loop while r4 != 0
	mov	r4,#$FF
LoopFF_06:
	djnz	r4,LoopFF_06	; r4--, loop while r4 != 0
;
	jt1	ReadHostLoop	; If T1 not zero after delay - key was bounced, false press detected, continue as not pressed
	;
	; Key is really pressed, debounced procedure confirm this. Do mark it as pressed.
	;
	call	GetBitByteIndex	; Get Byte index (offset) into PressRelease Bit Matrix in RAM for scan cycle depended key(r1)+bit number(r5)
	mov	a,@r1		; a = Byte index (offset) in PressRelease Bit Matrix in RAM	; a = RAM[r1]
	anl	a,r5		; Select according key into PressRelease Bit Matrix in RAM by masking all bits except our (r5)
	nop			; nop
	jnz	ReadHostLoop	; If not zero - already mark as pressed, no action need, continue as is
	mov	a,r6		; Get scan cycle from main
	jz	StrtKeyPresRept	; If nothing - first time repeat key detected - start key repeat mode
	xrl	a,r7		; Xor with scan cycle from timer
	jz	ContKeyPresRept	; If the same - key long pressed - continue key repeat mode
	stop	tcnt		; Else - stop timer, because other key pressed
	mov	a,r6		; Get scan cycle from main
	mov	r1,a		; r1 = scan cycle from main
	call	GetBitByteIndex	; Get Byte index (offset) into PressRelease Bit Matrix in RAM for scan cycle depended key(r1)+bit number(r5)
	mov	a,@r1		; a = Byte index (offset) in PressRelease Bit Matrix in RAM	; a = RAM[r1]
	orl	a,r5		; Mark according key into PressRelease Bit Matrix in RAM as Pressed
	mov	@r1,a		; RAM[r1] = a
;
StrtKeyPresRept:		; Start key repeat mode
	mov	r0,#Reg_r3a	; Get r3* address (Used in Timer)
	clr	a		; a = 0
	mov	@r0,a		; Clear r3* (Used in Timer)	; r3* = a	; RAM[r0] = a
	jmp	SetRegsAndSend	; Set both Main and Timer register, and r3 Scan Cycle register to new key value and send it to Host
;
PuToStickKeyArr:		; Put Scan Cycle in r3 to Sticked Keys Scan Codes Array
	mov	a,@r0		; a = StickKeysArr[r0]	; a = RAM[r0]
	jz	StickPlaceFound	;
	mov	a,r0		; Get index
	xrl	a,#$3F		; Max Memory ?
	jz	ReadHostLoop	; Continue general loop
	inc	r0		; Next Index
	jmp	PuToStickKeyArr	; Next attempt
;
StickPlaceFound:		; Free place found in Sticked Keys Codes Array
	mov	a,r3		; a = Sticked Key Code
	mov	@r0,a		; StickKeysArr[r0] = a	; RAM[r0] = a
	jf1	ReadHostLoop	; Continue general loop
	cpl	f1		; Invert f1 (???)
	jmp	ReadHostLoop	; Continue general loop
;
ContKeyPresRept:		; Continue of repeat long-term pressed key
	jf0	ReadHostLoop	; Continue general loop
	clr	a		; a = 0
	mov	t,a		; Timer = 0 (MaxTimer)
	strt	t		; Start Timer
	cpl	f0		; Invert f0 (???)
	jmp	ReadHostLoop	; Continue general loop
;
	nop			; nop
;
SendStickedKey:			; Send Stickey Keys to Host
	jmp	SendStickedKeys	; Jump helper to SendStickedKeys
;
	nop			; Allignment
	nop			; by nops
;
; r5 = byte to Host		; r5 = byte to Host
RealSndStickKey:		; This code really send Sticked Keys to Host
	;
	; Disable keyboard scan	; Disable keyboard scan
	;
	anl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = 0, Bit2 = ~Data OUT = X | ========> KX S0 DX
	mov	r4,#$07		; r4 = 07h (Bits number from 0)
	orl	p2,#$C3		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = X | ========> K1 S1 DX
NextBitToHost1:
	mov	a,r5		; r5 = byte to Host (Shifted right on iteration after first)
	jb0	Send1BitValue1	; Bit0 is 0 ? -> Send1BitValue1
	;
	; Send bit value 0	; Send bit value 0
	;
	anl	p2,#$81		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = 0, Bit2 = ~Data OUT = 0 | ========> KX S0 D0
	orl	p2,#$C3		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = X | ========> K1 S1 DX
RotateNextBit1:
	mov	a,r4		; a = r4 (initial 07h)
	jz	AllBitsSended1	; if (a == 0) AllBitsSended1
	dec	r4		; r4 = r4 - 1 (Initial 07h)
	mov	a,r5		; a = byte to Host (Shifted right on iteration after first)
	rr	a		; a = a >> 1
	mov	r5,a		; r5 = byte to Host (Shifted right on iteration after first)
	jmp	NextBitToHost1	; Go to sending next bit to Host
;
Send1BitValue1:			; Send bit value 1
	;
	; Send bit value 1	; Send bit value 1
	;
	orl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = X, Bit2 = ~Data OUT = 1 | ========> K1 SX D1
	anl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = 0, Bit2 = ~Data OUT = X | ========> KX S0 DX
	orl	p2,#$E7		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = 1 | ========> K1 S1 D1
	jmp	RotateNextBit1	; Go to processing next bit for send to Host
;
AllBitsSended1:			; Send stop bit (1), Data set to 0 after stop bit sended
	;
	; Send stop bit (1)	; Send stop bit (1), Data set to 0 after stop bit sended
	;
	orl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = X, Bit2 = ~Data OUT = 1 | ========> K1 SX D1
	anl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = 0, Bit2 = ~Data OUT = X | ========> KX S0 DX
	orl	p2,#$E7		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = 1 | ========> K1 S1 D1
	anl	p2,#$C3		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = X, Bit2 = ~Data OUT = 0 | ========> KX SX D0
;
	mov	r1,StickKeysArr+1 ; r1 = StickKeysArr+1 (next element)
ClrStickKeysArr:
	mov	a,@r1		; a = next element of StickKeysArr	; a = RAM[r1]
	jz	AllStckArrEmpty	; If zero than StickKeysArr empty now
	mov	@r0,a		; RAM[r0] = a
	inc	r0		; Increase Current Index
	inc	r1		; Increase Next Index
	mov	a,r1		; Get next StickKeysArr Element Index
	jb6	AllStckArrEmpty	; If maximal Index - All done
	jmp	ClrStickKeysArr	; Clear next element and move whole array
;
AllStckArrEmpty:		; Whole Stickers Keys Array is Empty
	clr	a		; a = 0
	mov	@r0,a		; Clear firs element of StickKeysArr	; RAM[r0] = a
	mov	r1,StickKeysArr	; r1 = StickKeysArr
	mov	a,@r1		; a = StickKeysArr[r1] (Scan Code)	; a = RAM[r1]
	jnz	ReadHostLoop	; Continue general loop
	clr	f1		; f1 = 0 (???)
	jmp	ReadHostLoop	; Continue general loop
;
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
;
DoSndStickKeys:			; Send Stickey Keys to Host
	orl	p2,#$07		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = 1 | ========> K1 S1 D1
	in	a,p1		; a = Port P1 Bit0 = Sync IN, Bit1 = Data IN, Bit5 = Rus LED, Bit6 = Lat LED, Bit7 = Cif LED
	jb1	SendStickedKey	; Send Sticked Keys to Host
	jmp	ChkTmrOverflow	; Check Timer Overflow and start it again
;
	nop			; nop
;
; === Page1ROM (256 Bytes) ===	; === Page1ROM (256 Bytes) ===
;
Phase2CRC:			; Calc CRC, Phase 2
	mov	a,r7		; r7 = Current CRC
CalcCRC2:
	movp	a,@a		; a = CurPageROM[a]
	add	a,r6		; a = a + r6
	mov	r6,a		; r6 = a         <= r6 = CRC
	inc	r7		; r7 = r7 + 1
	mov	a,r7		; a = r7
	jnz	CalcCRC2	; Loop
	jmp	Phase3CRC	; Go to next CRC calculation phase in Page2
;
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
;
TimerBegin:			; Real Timer Begin Entry Point
	;
	; After RAM test all RAM from 2 to 3FF (except r0 and r1) are cleared (= 0), i.e r2* = r3* = 0 on first timer start
	;
	mov	a,r3		; a = r3* (Initially 0)
	xrl	a,#$30		; a = a xor 30h
	jz	AllRowsScanned	; All rows scanned ? (bits 0-4)
	inc	r3		; r3* = r3* + 1
NextScanCycleT:
	sel	rb0		; Select back RAM Bank0 (registers w/o *)
	jmp	KbdScanCycleT	; Go to KbdScanCycleT - continue cycle of keyboard scan from timer
;
	nop			; nop
;
AllRowsScanned:			; All Rows Scanned
	mov	a,r2		; a = r2* (Initially 0)
	xrl	a,#$04		; a = a xor 04h
	jz	AllColsSelected	; All columns Selected (bits 0-2)
	inc	r2		; r2* = r2* + 1
	jmp	NextScanCycleT	; Go to NextScanCycleT (after RAM Bank switch to 0)
;
AllColsSelected:		; All Columns Selected
	clr	a		; a = 0
	mov	r2,a		; r2* = a
	sel	rb0		; Select back RAM Bank0 (registers w/o *)
	mov	a,r6		; Get scan cycle from main
	mov	r3,a		; Put it into r3
	jmp	TranslateToHost	; Translate Scan cycle in r3 register into Host Code. Bit7 is Lo for Press Code and Hi to Release Code.
;
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
;
;===========================================================================================================================================
; Return Byte index (offset) into PressRelease Bit Matrix in RAM for key, corresponding with ScanCycle (r1) and bit number in this byte (r5)
; Input: r1 = Scan cycle value where key press/release detected
; Return: r1 = Byte offset of requested key in PressRelease Bit Matrix
;         r5 = Bit into Byte in r1
GetBitByteIndex:
	sel	rb0		; Select RAM Bank0 (registers w/o *)
	dec	r1		; Dec Scan cycle
	mov	a,r1		; Save it to a
	mov	r4,a		; Save it to r4
	rr	a		; 
	rr	a		; a = a / 8 (Byte number)
	rr	a		; 
	anl	a,#$1F		; Mask low 5 bits - validate for 0 - 31 Max
	add	a,#PresRelMatr	; Add PressRelease Bit Matrix in RAM offset
	mov	r1,a		; Save to r1
	mov	r5,#$80		; r5 = Bit position into Byte
NextCycle:
	mov	a,r4		; a = Scan cycle (decremented in cycle from original)
	anl	a,#$07		; Mask low 3 bits (Validate 0-7 bit positions)
	mov	r4,a		; Save in r4
	jz	Return		; If a = 0 we found our bit position - return
	mov	a,r5		; a = Bit position into Byte
	rr	a		; Correct new Bit position into Byte
	mov	r5,a		; Save it back to r5
	dec	r4		; Decrement Scan cycle
	jmp	NextCycle	; Next cycle
Return:
	retr			; Ret
;===========================================================================================================================================
;
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
;
SetRegsAndSend:			; Set both Main and Timer register, and r3 Scan Cycle register to new key value and send it to Host
	mov	a,r7		; Get scan cycle from timer
	mov	r6,a		; Save it in scan cycle from main
	mov	r3,a		; Save it in scan cycle for Host

	clr	f0		; f0 = 0 (???)
;
TranslateToHost:		; Translate Scan cycle in r3 register into Host Code. Bit7 is Lo for Press Code and Hi to Release Code.
	;	
	; Translate Scan cycle in r3 register into Host Code. Bit7 is Lo for Press Code and Hi to Release Code.
	;
	mov	a,r3		; Get Scan Index for Released Key
	anl	a,#$80		; Mask Bit7 only = detect Press or Release (Bit7 = 1 -> Release Code to Host)
	mov	r0,a		; Save it into r0
	mov	a,r3		; Get Scan Index for Released Key
	anl	a,#$7F		; Mask Bits 0-6 only
	movp3	a,@a		; a = Traslated Scan Code to Host	; a = ScanCodes[a]	; a = Page3ROM[a]
	orl	a,r0		; Restore Bit7 (Bit7 = 1 -> Release Code to Host)
	mov	r3,a		; Save back to r3
	mov	r0,StickKeysArr	; r0 = StickKeysArr
	jmp	PuToStickKeyArr	; Put it into Sticked Keys Scan Codes Array
;
SendStickedKeys:		; Send Sticked Keys to Host
	mov	r0,StickKeysArr	; r0 = StickKeysArr
	mov	a,@r0		; a = StickKeysArr[r0] (Scan Codes)	; a = RAM[r0]
	mov	r5,a		; r5 = a (Byte to Host)
	xrl	a,#$57		; a = a xor 57h - Check if Scan Cycle set to 1/End key in matrix (Col 7, Row 6 [see schematic])
	jz	AllLedsOn2	; If yes - Lighth ALL LEDs and correct sync/data
	mov	a,r5		; Restore a
	xrl	a,#$5B		; a = a xor 5Bh - Check if Scan Cycle set to ./Del key in matrix (Col 12, Row 6 [see schematic])
	jz	AllLedsOn3	; If yes - Lighth ALL LEDs and correct sync/data
	mov	a,r5		; Restore a
	xrl	a,#$45		; a = a xor 45h - Check if Scan Cycle set to F1 key in matrix (Col 5, Row 5 [see schematic])
	jnz	RealSndStckKey	; Go to code which really send Sticked Keys to Host (r5 = Byte to Host)
	jf0	RealSndStckKey	; Go to code which really send Sticked Keys to Host (r5 = Byte to Host)
	mov	a,r2
	jz	AllLedsOn1	; If yes - Lighth ALL LEDs and correct sync/data
	mov	r2,#$00
	anl	p1,#$7F		; Sync IN = X, Data IN = X | LED : RX LX CX
	jmp	RealSndStickKey	; Go to code which really send Sticked Keys to Host (r5 = Byte to Host)
;
AllLedsOn1:			; All LEDs on, Sync1, DataX
	mov	r2,#$01
	orl	p1,#$9F		; Sync IN = 1, Data IN = X | LED : R1 L1 C1
	jmp	RealSndStickKey	; Go to code which really send Sticked Keys to Host (r5 = Byte to Host)
;
AllLedsOn2:			; All LEDs on, Sync1, Data0-X
	anl	p1,#$9F		; Sync IN = X, Data IN = 0 | LED : RX LX CX
	orl	p1,#$5F		; Sync IN = 1, Data IN = X | LED : R1 L1 C1
	jmp	RealSndStickKey	; Go to code which really send Sticked Keys to Host (r5 = Byte to Host)
;
AllLedsOn3:			; All LEDs on, Sync1, Data0-1
	anl	p1,#$9F		; Sync IN = X, Data IN = 0 | LED : RX LX CX
	orl	p1,#$3F		; Sync IN = 1, Data IN = 1 | LED : R1 L1 C1
RealSndStckKey:
	jmp	RealSndStickKey	; Go to code which really send Sticked Keys to Host (r5 = Byte to Host)
;
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00		; Allignment by nops
;
CRC:	.db	$A5		; CRC - Allignment sum of all bytes to 0
;
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
;
RamTest2:			; RAM Test, phase 2
	cpl	f1		; Flop f1 (Indicate third phase)
	mov	r1,#$02		; r1 = 02h (Starting RAM address 02h for test)
	mov	r0,#$00		; r0 = 000h (Testing Mask)
	jmp	RamTest1	; Go to next phase of RAM test
;
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00		; Allignment by nops
;
; === Page2ROM (256 Bytes) ===	; === Page2ROM (256 Bytes) ===
;
Phase3CRC:			; CRC, Phase 3
	mov	a,r7		; r7 = Current CRC
CalcCRC3:
	movp	a,@a		; a = CurPageROM[a]
	add	a,r6		; a = a + r6
	mov	r6,a		; r6 = a         <= r6 = CRC
	inc	r7		; r7 = r7 + 1
	mov	a,r7		; a = r7
	jnz	CalcCRC3	; Loop
CalcCRC4:
	movp3	a,@a		; a = Page3ROM[a] (ScanCodes[a])
	add	a,r6		; a = a + r6
	mov	r6,a		; r6 = a         <= r6 = CRC
	inc	r7		; r7 = r7 + 1
	mov	a,r7		; a = r7
	jnz	CalcCRC4	; Loop
	mov	a,r6		; a = CRC
	jnz	BadCRC		; BAD CRC
TestR0:
	mov	r0,#$AA		; r0 = 0AAh
	mov	a,#$AA		; a = 0AAh
	xrl	a,r0		; a = 0 (a xor r0)
	jnz	TestError
	mov	r0,#$55		; r0 = 055h
	mov	a,#$55		; a = 055h
	xrl	a,r0		; a = 0 (a xor r0)
	jnz	TestError	; Go to test error
	jmp	TestR1		; Go to test next register - r1
;
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
TestR1:
	mov	r1,#$AA		; r1 = 0AAh
	mov	a,#$AA		; a = 0AAh
	xrl	a,r1		; a = 0 (a xor r0)
	jnz	TestError
	mov	r1,#$55		; r1 = 055h
	mov	a,#$55		; a = 055h
	xrl	a,r1		; a = 0 (a xor r0)
	jnz	TestError
	mov	r1,#$02		; r1 = 02h (Starting RAM address 02h for test)
	mov	r0,#$AA		; r0 = 0AAh (Testing Mask)

; r0 = Testing Mask		; r0 = Testing Mask
; r1 = Started addr for RAM test; r1 = Started addr for RAM test
RamTest1:
	mov	a,r0		; a = 0AAh
	mov	@r1,a		; RAM[r1] = a
	mov	a,@r1		; a = RAM[r1]
	xrl	a,r0		; a = 0 (a xor r0)
	jnz	TestError	; Non Zero - Error
	inc	r1		; r1 = r1 + 1 (Nex RAM address)
	mov	a,r1		; a = r1
	xrl	a,#$40		; a = a xor 040h (64) - is Max RAM address reached ?
	jnz	RamTest1	; Loop on whole RAM
	jf0	RamTstPhase1Done; Exit if one phase done
	cpl	f0		; Flop f0 (Indicate 2nd pass of RAM test)
	mov	r1,#$02		; r1 = 02h (Starting RAM address 02h for test)
	mov	r0,#$55		; r0 = 055h (Testing Mask)
	jmp	RamTest1	; Go to test again with new mask
;
RamTstPhase1Done:		; RAM Test Phase1 Done
	jf1	RamTstEndedOK	; All three phases of RAM test done ?
	jmp	RamTest2	; No - goto second phase
;
RamTstEndedOK:			; RAM Test Ended OK
	;
	; All RAM from 2 to 3FF (except r0 and r1) are cleared (= 0), f0 = f1 = 1
	;
	mov	r7,#$AA		; r7 = 0AAh (Confirm "Test OK" code, returned to host)
;
; r7 = byte to Host		; r7 = byte to Host
SendKeyToHost:			; Send Key to Host
	;
	; Send start bit (1)	; Send start bit (1)
	;
	orl	p2,#$E7		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = 1 | ========> K1 S1 D1
	in	a,p1		; a = Port P1 Bit0 = Sync IN, Bit1 = Data IN, Bit5 = Rus LED, Bit6 = Lat LED, Bit7 = Cif LED
	cpl	a		; a = NOT a
	jb0	ErrSyncHi	; Bit0 (Sync IN) = 0 ? ErrSyncHi
	jb1	SendKeyToHost	; Bit1 (Data IN) = 0 ? SendKeyToHost
	anl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = 0, Bit2 = ~Data OUT = X | ========> KX S0 DX
	mov	r4,#$07		; r4 = 07h (Bits number from 0)
	orl	p2,#$C3		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = X | ========> K1 S1 DX
NextBitToHost2:
	mov	a,r7		; r7 = byte to Host (Shifted right on iteration after first)
	jb0	Send2BitValue1	; Bit0 is 0 ? -> Send2BitValue1
	;
	; Send bit value 0	; Send bit value 0
	;
	anl	p2,#$81		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = 0, Bit2 = ~Data OUT = 0 | ========> KX S0 D0
	orl	p2,#$C3		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = X | ========> K1 S1 DX
RotateNextBit2:
	mov	a,r4		; a = r4 (initial 07h)
	jz	AllBitsSended2	; if (a == 0) AllBitsSended2 
	dec	r4		; r4 = r4 - 1 (Initial 07h)
	mov	a,r7		; a = byte to Host (Shifted right on iteration after first)
	rr	a		; a = a >> 1
	mov	r7,a		; r7 = byte to Host (Shifted right on iteration after first)
	jmp	NextBitToHost2	; Go to sending next bit to Host
;
ErrSyncHi:			; Sync Error
	in	a,p1		; a = Port P1 Bit0 = Sync IN, Bit1 = Data IN, Bit5 = Rus LED, Bit6 = Lat LED, Bit7 = Cif LED
	cpl	a		; a = NOT a
	jb0	ErrSyncHi	; Bit0 (Sync IN) = 0 ? ErrSyncHi
	jmp	Start		; Go to Start
;
Send2BitValue1:			; Send bit value 1
	;
	; Send bit value 1	; Send bit value 1
	;
	orl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = X, Bit2 = ~Data OUT = 1 | ========> K1 SX D1
	anl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = 0, Bit2 = ~Data OUT = X | ========> KX S0 DX
	orl	p2,#$E7		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = 1 | ========> K1 S1 D1
	jmp	RotateNextBit2	; Go to processing next bit for send to Host
;
AllBitsSended2:			; Send stop bit (1), Data set to 0 after stop bit sended
	;
	; Send stop bit (1)	; Send stop bit (1), Data set to 0 after stop bit sended
	;
	orl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = X, Bit2 = ~Data OUT = 1 | ========> K1 SX D1
	anl	p2,#$A5		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = 0, Bit2 = ~Data OUT = X | ========> KX S0 DX
	orl	p2,#$E7		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 1, Bit1 = ~Sync OUT = 1, Bit2 = ~Data OUT = 1 | ========> K1 S1 D1
	anl	p2,#$C3		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = X, Bit1 = ~Sync OUT = X, Bit2 = ~Data OUT = 0 | ========> KX SX D0
	jf0	DoNewScanCycleM	; After Testing (POST) f0 = 1 -> Go to DoNewScanCycleM (First cycle, else - continue)
KbdScanCycleM:
	inc	r6		; Next scan cycle from main
	mov	a,r6		; a = r6
	cpl	a		; a = NOT a (Max scan cycle from main ?)
	jb7	ContinScanCyclM	; Continue scan cycle from main
	;
	; First keyboard scan cycle from main done -> Reset all cycle registers (a = r1 = r6 = r7 = f0 = 0) and go to read Host
	;
	clr	a		; a = 0
	mov	r6,a		; r6 = 0
	mov	r7,a		; r7 = 0
	mov	r1,a		; r1 = 0
	clr	f0		; Clear f0 (flag0)
	jmp	ReadHostPrepare	; Go to read Host (General Loop "ReadHostLoop" after prepare)
;
TestError:			; Test error - restart from begiinning after some checks
	mov	a,#$5F		; Activate column 16 (bits 0-3), row 6 (bits 4-6) (See schematic) - unused key in matrix
	outl	bus,a		; Select this unused key
	jnt0	RamTstPhase1Done; t0 not used according to schematic - RamTstPhase1Done. May be debug feature by manufacturer when t0 is used by plant to continue with bad ROM..
	jmp	TestR0		; Do All Again
;
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
;
DoNewScanCycleM:		; Do new Scan Cycle
	mov	r6,#$01		; Keyboard scan cycle from main reset to 1
ContinScanCyclM:
	mov	a,r6		; Get scan cycle from main
	outl	bus,a		; Out it to bus
	;
	; Enable keyboard scan	; Enable keyboard scan
	;
	anl	p2,#$42		; Port P2 Bit0 = ~Kbd Matrix Scan Enable = 0, Bit1 = ~Sync OUT = X, Bit2 = ~Data OUT = 0 | ========> K0 SX D0

	clr	f1		; f1 = 0 (???)
	;
	; Key press detecting	; Key press detecting
	;
	jt1	KbdScanCycleM	; If no key pressed - next scan cycle from main
	;
	; Some key pressed	; Some key pressed
	;
	mov	a,r6		; Get scan cycle from main
	mov	r1,a		; r1 = scan cycle from main
	call	GetBitByteIndex	; Get Byte index (offset) into PressRelease Bit Matrix in RAM for scan cycle depended key(r1)+bit number(r5)
	mov	a,@r1		; a = Byte index (offset) in PressRelease Bit Matrix in RAM	; a = RAM[r1]
	orl	a,r5		; Mark according key into PressRelease Bit Matrix in RAM as Pressed
	mov	@r1,a		; Store it back into PressRelease Bit Matrix in RAM		; RAM[r1] = a
	jf0	TransSndScanCod	; If flag that need Send to Host = 1 => Translate To ScanCode and Send to Host
	jmp	KbdScanCycleM	; Next scan cycle from main
;
	nop			; Allignment
	nop			; by nops
;
BadCRC:				; Bad CRC
	outl	bus,a		; Out bad CRC to bus (Port P0). May be debug feature by manufacturer when t0 is used by plant.
DeadLoop:
	jnt0	TestR0		; t0 not used according to schematic - deadloop. May be debug feature by manufacturer when t0 is used by plant to continue with bad ROM..
	jmp	DeadLoop	; Dead loop
;
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
	nop			; Allignment by nops
;
TransSndScanCod:		; Translatye and send Key to Host
	;
	; Translate to Scan Code and send Scan Code to Host
	;	
	clr	f0		; Set flag that byte already sended to Host
	mov	a,r6		; Get scan cycle from main
	movp3	a,@a		; Traslate it to Scan Code to Host	; a = ScanCodes[a]	; a = Page3ROM[a]
	mov	r7,a		; r7 = byte to Host
	jmp	SendKeyToHost	; Go to Send Byte to Host
;
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00	; Allignment by nops
;
; === Page3ROM (256 Bytes) ===	; === Page3ROM (256 Bytes) ===
;
ScanCodes:			; Scan Codes
	.db	$00,$01,$02,$03	; 00 000
	.db	$04,$05,$06,$07	; 04 004
	.db	$08,$09,$0A,$0B	; 08 008
	.db	$0C,$0D,$2B,$0E	; 0C 012
	.db	$0F,$10,$11,$12	; 10 016
	.db	$13,$14,$15,$16	; 14 020
	.db	$17,$18,$19,$36	; 18 024
	.db	$3A,$35,$37,$1D	; 1C 028
	.db	$1E,$1F,$20,$21	; 20 032
	.db	$22,$23,$24,$25	; 24 036
	.db	$26,$1A,$1B,$27	; 28 040
	.db	$28,$1C,$38,$54	; 2C 044  | Summary 93 bytes size, corresponding with table in both ES1840.E000 descriptions i have.
	.db	$2C,$2D,$2E,$2F	; 30 048
	.db	$30,$31,$32,$29	; 34 052
	.db	$2A,$5C,$33,$34	; 38 056
	.db	$55,$56,$57,$58	; 3C 060
	.db	$59,$39,$5A,$5B	; 40 064
	.db	$3B,$3C,$3D,$3E	; 44 068
	.db	$3F,$40,$41,$42	; 48 072
	.db	$43,$44,$45,$46	; 4C 076
	.db	$4A,$47,$48,$49	; 50 080
	.db	$4B,$4C,$4D,$4F	; 54 084
	.db	$50,$51,$52,$53	; 58 088
	.db	$4E		; 5C 092
;
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00,00,00	; Allignment by nops
	.db	00,00,00	; Allignment by nops
;
; === END OF 1KB ROM ===	; Done
;
